@@ -1,4 +1,4 @@
-public :: grib_get, grib_set, grib_get_data
+public :: grib_get, grib_set, grib_get_data, grib_is_missing
 public :: grib_open_file, grib_close_file
 public :: grib_multi_support_on, grib_multi_support_off
 public :: grib_keys_iterator_new, &
@@ -21,6 +21,6 @@ public :: grib_get_size
 public :: grib_get_message_size, grib_copy_message
 public :: grib_write, grib_multi_append
 public :: grib_check
-public :: grib_clone
+public :: grib_clone, grib_copy_namespace
 public :: grib_index_get,grib_index_select,&
           grib_index_create,grib_index_get_size,grib_index_release
