@@ -279,7 +279,7 @@ module grib_api
       if (present(status)) then
          status = iret
       else
-         call grib_check(iret,'grib_set_missing',key)
+         call grib_check(iret,'grib_set_missing','('//key//')')
       endif
   end subroutine grib_set_missing
 
@@ -307,7 +307,7 @@ module grib_api
     if (present(status)) then
     status = iret
     else
-        call grib_check(iret,'grib_create_index',filename)
+        call grib_check(iret,'grib_create_index','('//filename//')')
     endif
       
   end subroutine grib_index_create
@@ -336,7 +336,7 @@ module grib_api
     if (present(status)) then
       status = iret
     else
-      call grib_check(iret,'grib_index_get_size',key)
+      call grib_check(iret,'grib_index_get_size','('//key//')')
     endif
 
   end subroutine grib_index_get_size_long
@@ -365,7 +365,7 @@ module grib_api
     if (present(status)) then
     status = iret
     else
-    call grib_check(iret,'grib_index_get_size',key)
+    call grib_check(iret,'grib_index_get_size','('//key//')')
     endif
 
   end subroutine grib_index_get_size_int
@@ -395,7 +395,7 @@ module grib_api
     if (present(status)) then
     status = iret
     else
-    call grib_check(iret,'grib_index_get',key)
+    call grib_check(iret,'grib_index_get','('//key//')')
     endif
   end subroutine grib_index_get_int
       
@@ -424,7 +424,7 @@ module grib_api
     if (present(status)) then
     status = iret
     else
-    call grib_check(iret,'grib_index_get',key)
+    call grib_check(iret,'grib_index_get','('//key//')')
     endif
   end subroutine grib_index_get_long
 
@@ -453,7 +453,7 @@ module grib_api
     if (present(status)) then
     status = iret
     else
-    call grib_check(iret,'grib_index_get',key)
+    call grib_check(iret,'grib_index_get','('//key//')')
     endif
   end subroutine grib_index_get_real8
 
@@ -481,7 +481,7 @@ module grib_api
     if (present(status)) then
     status = iret
     else
-      call grib_check(iret,'grib_index_select',key)
+      call grib_check(iret,'grib_index_select','('//key//')')
     endif
       end subroutine grib_index_select_int
       
@@ -509,7 +509,7 @@ module grib_api
       if (present(status)) then
       status = iret
       else
-      call grib_check(iret,'grib_index_select',key)
+      call grib_check(iret,'grib_index_select','('//key//')')
       endif
   end subroutine grib_index_select_long
       
@@ -537,7 +537,7 @@ module grib_api
     if (present(status)) then
     status = iret
     else
-    call grib_check(iret,'grib_index_select',key)
+    call grib_check(iret,'grib_index_select','('//key//')')
     endif
     end subroutine grib_index_select_real8
 
@@ -613,7 +613,7 @@ module grib_api
         if (present(status)) then
            status = iret
         else
-           call grib_check(iret,'grib_open_file',filename)
+           call grib_check(iret,'grib_open_file','('//filename//')')
         endif
   end subroutine grib_open_file
 
@@ -641,7 +641,7 @@ module grib_api
         if (present(status)) then
            status = iret
         else
-           call grib_check(iret,'grib_pbopen',filename)
+           call grib_check(iret,'grib_pbopen','('//filename//')')
         endif
   end subroutine grib_pbopen
 
@@ -800,7 +800,7 @@ module grib_api
       if (present(status)) then
          status = iret
       else
-         call grib_check(iret,'grib_new_from_samples','')
+         call grib_check(iret,'grib_new_from_samples','('//samplename//')')
       endif
   end subroutine grib_new_from_samples
 
@@ -832,7 +832,7 @@ module grib_api
       if (present(status)) then
          status = iret
       else
-         call grib_check(iret,'grib_new_from_template','')
+         call grib_check(iret,'grib_new_from_template','('//templatename//')')
       endif
   end subroutine grib_new_from_template
 
@@ -884,10 +884,39 @@ module grib_api
       if (present(status)) then
          status = iret
       else
-         call grib_check(iret,'grib_release','')
+         call grib_check(iret,'grib_clone','')
       endif
   end subroutine grib_clone
 
+  !> Copy the value of all the keys belonging to a namespace from the source message
+  !> to the destination message
+  !>
+  !>
+  !> In case of error, if the status parameter (optional) is not given, the program will
+  !> exit with an error message.\n Otherwise the error message can be
+  !> gathered with @ref grib_get_error_string.
+  !>
+  !>
+  !>
+  !> @param gribid_src     source message
+  !> @param gribid_dest    destination message
+  !> @param namespace      namespace to be copied
+  !> @param status         GRIB_SUCCESS if OK, integer value on error
+  subroutine grib_copy_namespace ( gribid_src, namespace, gribid_dest, status )
+      integer(kind=kindOfInt),          intent(in)  :: gribid_src
+      integer(kind=kindOfInt),          intent(in)  :: gribid_dest
+      character(LEN=*),                 intent(in)  :: namespace
+      integer(kind=kindOfInt),optional, intent(out) :: status
+      integer(kind=kindOfInt)                       :: iret
+
+      iret=grib_f_copy_namespace(gribid_src,namespace,gribid_dest)
+      if (present(status)) then
+         status = iret
+      else
+         call grib_check(iret,'grib_copy_namespace','('//namespace//')')
+      endif
+  end subroutine grib_copy_namespace
+
   !> Check the status returned by a subroutine.
   !>
   !> In case of error it stops the program, returns the error code to the shell
@@ -1242,6 +1271,31 @@ module grib_api
     endif
     end subroutine grib_get_long
 
+  !> Check if the value of a key is MISSING.
+  !>
+  !> In case of error, if the status parameter (optional) is not given, the program will
+  !> exit with an error message.\n Otherwise the error message can be
+  !> gathered with @ref grib_get_error_string.
+  !>
+  !> @param gribid      id of the grib loaded in memory
+  !> @param key         key name
+  !> @param is_missing  0->not missing, 1->missing
+  !> @param status      GRIB_SUCCESS if OK, integer value on error
+  subroutine grib_is_missing(gribid,key,is_missing,status)
+  integer(kind=kindOfInt),          intent(in)  :: gribid
+  character(len=*), intent(in)                  :: key
+  integer(kind = kindOfInt),     intent(out)    :: is_missing
+  integer(kind=kindOfInt),optional, intent(out) :: status
+  integer(kind=kindOfInt)                       :: iret
+
+    iret=grib_f_is_missing ( gribid, key, is_missing )
+    if (present(status)) then
+    status = iret
+    else
+    call grib_check(iret,'grib_is_missing',key)
+    endif
+    end subroutine grib_is_missing
+
 
   !> Get the real(4) value of a key from a grib message.
   !>
