@@ -1139,6 +1139,23 @@ int grib_f_clone(int* gidsrc,int* giddest){
   return grib_f_clone_(gidsrc, giddest);
 }
 
+int grib_f_copy_namespace_(int* gidsrc,char* name,int* giddest,int len){
+	char buf[1024]={0,};
+	grib_handle *src  = get_handle(*gidsrc);
+	grib_handle *dest = get_handle(*giddest);
+
+	if(src && dest)
+		return grib_copy_namespace(dest,cast_char(buf,name,len),src);
+	
+	return GRIB_INVALID_GRIB;
+}
+int grib_f_copy_namespace__(int* gidsrc,char* name,int* giddest,int len){
+	return grib_f_copy_namespace_(gidsrc,name,giddest,len);
+}
+int grib_f_copy_namespace(int* gidsrc,char* name,int* giddest,int len){
+	return grib_f_copy_namespace_(gidsrc,name,giddest,len);
+}
+
 int grib_f_count_in_file(int* fid,int* n) {
   int err = 0;
   FILE* f = get_file(*fid);
@@ -1208,7 +1225,7 @@ int grib_f_new_from_index(int* iid, int* gid){
   return grib_f_new_from_index_(iid,gid);
 }
 
-int grib_f_index_new_from_file_(char* file ,                                char* keys ,int* gid, int lfile, int lkeys){
+int grib_f_index_new_from_file_(char* file ,char* keys ,int* gid, int lfile, int lkeys){
   int err = 0;
   char fname[1024]={0,};
   char knames[1024]={0,};
@@ -1664,6 +1681,21 @@ int grib_f_set_missing(int* gid, char* key,  int len){
   return grib_f_set_missing_( gid,  key,   len);
 }
 
+int grib_f_is_missing_(int* gid, char* key,int* isMissing,int len){
+    int err=0;
+	grib_handle *h = get_handle(*gid);
+	char buf[1024];
+	if(!h)  return GRIB_INVALID_GRIB;
+
+	*isMissing=grib_is_missing(h, cast_char(buf,key,len),&err);
+	return err;
+}
+int grib_f_is_missing__(int* gid, char* key,int* isMissing,int len){
+	return grib_f_is_missing_(gid,key,isMissing,len);
+}
+int grib_f_is_missing(int* gid, char* key,int* isMissing,int len){
+	return grib_f_is_missing_(gid,key,isMissing,len);
+}
 
 int grib_f_set_real4_(int* gid, char* key, float* val, int len){
 
