@@ -78,6 +78,9 @@ int grib_f_new_from_template(int *gid, char *name, int lname);
 int grib_f_clone_(int *gidsrc, int *giddest);
 int grib_f_clone__(int *gidsrc, int *giddest);
 int grib_f_clone(int *gidsrc, int *giddest);
+int grib_f_copy_namespace_(int *gidsrc, char *name, int *giddest, int len);
+int grib_f_copy_namespace__(int *gidsrc, char *name, int *giddest, int len);
+int grib_f_copy_namespace(int *gidsrc, char *name, int *giddest, int len);
 int grib_f_count_in_file(int *fid, int *n);
 int grib_f_count_in_file_(int *fid, int *n);
 int grib_f_count_in_file__(int *fid, int *n);
@@ -93,6 +96,9 @@ int grib_f_index_new_from_file(char *file, char *keys, int *gid, int lfile, int
 int grib_f_index_release_(int *hid);
 int grib_f_index_release__(int *hid);
 int grib_f_index_release(int *hid);
+int grib_f_multi_handle_release_(int *hid);
+int grib_f_multi_handle_release__(int *hid);
+int grib_f_multi_handle_release(int *hid);
 int grib_f_release_(int *hid);
 int grib_f_release__(int *hid);
 int grib_f_release(int *hid);
@@ -153,6 +159,9 @@ int grib_f_set_long(int *gid, char *key, long *val, int len);
 int grib_f_set_missing_(int *gid, char *key, int len);
 int grib_f_set_missing__(int *gid, char *key, int len);
 int grib_f_set_missing(int *gid, char *key, int len);
+int grib_f_is_missing_(int *gid, char *key, int *isMissing, int len);
+int grib_f_is_missing__(int *gid, char *key, int *isMissing, int len);
+int grib_f_is_missing(int *gid, char *key, int *isMissing, int len);
 int grib_f_set_real4_(int *gid, char *key, float *val, int len);
 int grib_f_set_real4__(int *gid, char *key, float *val, int len);
 int grib_f_set_real4(int *gid, char *key, float *val, int len);
