@@ -82,7 +82,8 @@ static grib_accessor_class _grib_accessor_class_array = {
     0,      /* nearest_smaller_value */
     0,                       /* next accessor    */
     0,                    /* compare vs. another accessor   */
-    0,             /* unpack only ith value          */
+    0,     /* unpack only ith value          */
+    0,             		/* clear          */
 };
 
 
@@ -116,6 +117,7 @@ static void init_class(grib_accessor_class* c)
 	c->next	=	(*(c->super))->next;
 	c->compare	=	(*(c->super))->compare;
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
+	c->clear	=	(*(c->super))->clear;
 }
 /* END_CLASS_IMP */
 
