@@ -60,6 +60,9 @@ typedef struct grib_accessor_data_complex_packing {
 	const char* offsetsection;
 	int dirty;
 /* Members defined in data_simple_packing */
+	const char*  units_factor;
+	const char*  units_bias;
+	const char*  changing_precision;
 	const char*  number_of_values;
 	const char*  bits_per_value;
 	const char*  reference_value;
@@ -114,7 +117,8 @@ static grib_accessor_class _grib_accessor_class_data_complex_packing = {
     0,      /* nearest_smaller_value */
     0,                       /* next accessor    */
     0,                    /* compare vs. another accessor   */
-    0,             /* unpack only ith value          */
+    0,     /* unpack only ith value          */
+    0,             		/* clear          */
 };
 
 
@@ -146,6 +150,7 @@ static void init_class(grib_accessor_class* c)
 	c->next	=	(*(c->super))->next;
 	c->compare	=	(*(c->super))->compare;
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
+	c->clear	=	(*(c->super))->clear;
 }
 /* END_CLASS_IMP */
 
@@ -187,7 +192,10 @@ static long value_count(grib_accessor* a)
   if((ret = grib_get_long_internal(a->parent->h,self->pen_k,&pen_k)) != GRIB_SUCCESS)                         return ret;
   if((ret = grib_get_long_internal(a->parent->h,self->pen_m,&pen_m)) != GRIB_SUCCESS)                         return ret;
 
-  Assert ((pen_j ==  pen_k) && (pen_j == pen_m));
+  if (pen_j != pen_k || pen_j!=pen_m ) {
+    grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,"pen_j=%ld, pen_k=%ld, pen_m=%ld\n",pen_j,pen_k,pen_m);
+  	Assert ((pen_j ==  pen_k) && (pen_j == pen_m));
+  }
   return  (pen_j+1)*(pen_j+2);
 }
 
@@ -622,8 +630,6 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len) {
     return GRIB_INTERNAL_ERROR;
   }
 
-  if(laplacianOperatorIsSet == 0)
-
   if(!laplacianOperatorIsSet) {
     laplacianOperator = calculate_pfactor(a->parent->h->context,val,pen_j,sub_j);
     if((ret = grib_set_double_internal(a->parent->h,self->laplacianOperator,laplacianOperator))
