@@ -8,6 +8,7 @@
 */
 
 #include "grib_api_internal.h"
+#define PNG_ANYBITS
 
 /*
    This is used by make_class.pl
@@ -108,7 +109,8 @@ static grib_accessor_class _grib_accessor_class_data_png_packing = {
     0,      /* nearest_smaller_value */
     0,                       /* next accessor    */
     0,                    /* compare vs. another accessor   */
-    0,             /* unpack only ith value          */
+    0,     /* unpack only ith value          */
+    0,             		/* clear          */
 };
 
 
@@ -140,6 +142,7 @@ static void init_class(grib_accessor_class* c)
 	c->next	=	(*(c->super))->next;
 	c->compare	=	(*(c->super))->compare;
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
+	c->clear	=	(*(c->super))->clear;
 }
 /* END_CLASS_IMP */
 
@@ -222,18 +225,19 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
   long decimal_scale_factor = 0;
   double reference_value = 0;
   long bits_per_value =0;
+  long bits8;
 
   png_structp png  = 0;
   png_infop info = 0,end = 0;
   png_bytepp rows = 0;
   int interlace = 0,colour = 0,compression = 0,filter = 0,depth = 0;
 
-  self->dirty=0;
-
   png_uint_32 width = 0,height = 0;
 
   png_read_callback_data callback_data;
 
+  self->dirty=0;
+
   n_vals = grib_value_count(a);
 
   if((err = grib_get_long_internal(a->parent->h,self->bits_per_value,&bits_per_value)) != GRIB_SUCCESS)
@@ -322,9 +326,13 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
 
   if (colour == PNG_COLOR_TYPE_RGB)      depth=24;
   if (colour == PNG_COLOR_TYPE_RGB_ALPHA) depth=32;
+  bits8 = ((bits_per_value+7)/8)*8;
 
-
-  Assert(depth == bits_per_value);
+#ifdef PNG_ANYBITS
+  Assert(depth == bits8);
+#else
+  Assert( bits_per_value % 8 == 0 );
+#endif
 
   i = 0;
 
@@ -336,7 +344,7 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
     long pos = 0;
     int k;
     for(k = 0 ; k < width; k++)
-      val[i++] = (double) (((grib_decode_unsigned_long(row, &pos, bits_per_value)*bscale)+reference_value)*dscale);
+      val[i++] = (double) (((grib_decode_unsigned_long(row, &pos, bits8)*bscale)+reference_value)*dscale);
   }
   /*-------------------------------------------*/
   *len = n_vals;
@@ -419,6 +427,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
       /* Make sure we can decode it again */
       double ref = 1e-100;
       grib_get_double_internal(a->parent->h,self->reference_value,&ref);
+	  printf("%g %g %g\n",reference_value,ref,reference_value-ref);
       Assert(ref == reference_value);
     }
 
@@ -508,7 +517,9 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
   binary_scale_factor = grib_get_binary_scale_fact(max,reference_value,bits_per_value);
   divisor = grib_power(-binary_scale_factor,2);
 
+#ifndef PNG_ANYBITS
   Assert(bits_per_value % 8 == 0);
+#endif
   bits8 = (bits_per_value+7)/8*8;
   encoded = grib_context_buffer_malloc_clear(a->parent->h->context,bits8/8*n_vals);
 
@@ -591,16 +602,16 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 
   png_set_write_fn(png,&callback_data,png_write_callback,png_flush_callback);
 
-  depth = bits_per_value;
+  depth = bits8;
 
   colour = PNG_COLOR_TYPE_GRAY;
-  if (bits_per_value == 24)
+  if (bits8 == 24)
   {
     depth=8;
     colour=PNG_COLOR_TYPE_RGB;
   }
 
-  if(bits_per_value == 32 )
+  if(bits8 == 32 )
   {
     depth=8;
     colour=PNG_COLOR_TYPE_RGB_ALPHA;
@@ -612,7 +623,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 
 
   /*bytes=bit_depth/8;*/
-  bytes = bits_per_value/8;
+  bytes = bits8/8;
 
   rows = grib_context_buffer_malloc_clear(a->parent->h->context,sizeof(png_bytep)*height);
 
