@@ -24,6 +24,9 @@
    IMPLEMENTS = unpack_double_element
    IMPLEMENTS = pack_double
    IMPLEMENTS = value_count
+   MEMBERS=const char*  units_factor
+   MEMBERS=const char*  units_bias  
+   MEMBERS=const char*  changing_precision  
    MEMBERS=const char*  number_of_values
    MEMBERS=const char*  bits_per_value
    MEMBERS=const char*  reference_value
@@ -60,6 +63,9 @@ typedef struct grib_accessor_data_simple_packing {
 	const char* offsetsection;
 	int dirty;
 /* Members defined in data_simple_packing */
+	const char*  units_factor;
+	const char*  units_bias;
+	const char*  changing_precision;
 	const char*  number_of_values;
 	const char*  bits_per_value;
 	const char*  reference_value;
@@ -103,7 +109,8 @@ static grib_accessor_class _grib_accessor_class_data_simple_packing = {
     0,      /* nearest_smaller_value */
     0,                       /* next accessor    */
     0,                    /* compare vs. another accessor   */
-    &unpack_double_element,             /* unpack only ith value          */
+    &unpack_double_element,     /* unpack only ith value          */
+    0,             		/* clear          */
 };
 
 
@@ -134,13 +141,16 @@ static void init_class(grib_accessor_class* c)
 	c->nearest_smaller_value	=	(*(c->super))->nearest_smaller_value;
 	c->next	=	(*(c->super))->next;
 	c->compare	=	(*(c->super))->compare;
+	c->clear	=	(*(c->super))->clear;
 }
 /* END_CLASS_IMP */
 
 static void init(grib_accessor* a,const long v, grib_arguments* args)
 {
   grib_accessor_data_simple_packing *self =(grib_accessor_data_simple_packing*)a;
-
+  self->units_factor  = grib_arguments_get_name(a->parent->h,args,self->carg++);
+  self->units_bias  = grib_arguments_get_name(a->parent->h,args,self->carg++);
+  self->changing_precision  = grib_arguments_get_name(a->parent->h,args,self->carg++);
   self->number_of_values  = grib_arguments_get_name(a->parent->h,args,self->carg++);
   self->bits_per_value  = grib_arguments_get_name(a->parent->h,args,self->carg++);
   self->reference_value = grib_arguments_get_name(a->parent->h,args,self->carg++);
@@ -154,19 +164,13 @@ static void init(grib_accessor* a,const long v, grib_arguments* args)
 static long value_count(grib_accessor* a){
   grib_accessor_data_simple_packing* self =  (grib_accessor_data_simple_packing*)a;
 
-  long   bits_per_value;
   long number_of_values;
 
-   if(grib_get_long_internal(a->parent->h,self->bits_per_value,&bits_per_value) != GRIB_SUCCESS)
-    return 0;
-
-   /*if (bits_per_value == 0) {*/
-     if(grib_get_long_internal(a->parent->h,self->number_of_values,&number_of_values) != GRIB_SUCCESS)
+  if(grib_get_long_internal(a->parent->h,self->number_of_values,&number_of_values) != GRIB_SUCCESS)
        return 0;
-     return number_of_values;
-   /*}*/
-
-   /*return (a->length*8)/bits_per_value;*/
+  
+  return number_of_values;
+  
 }
 
 static int  unpack_double_element(grib_accessor* a, size_t idx, double* val)
@@ -279,19 +283,36 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
   double s = 0;
   double d = 0;
   long pos = 0;
+  double units_factor=1.0;
+  double units_bias=0.0;
 
   n_vals = grib_value_count(a);
 
+  if(*len < n_vals)
+  {
+    *len = (long) n_vals;
+    return GRIB_ARRAY_TOO_SMALL;
+  }
+
   if((err = grib_get_long_internal(a->parent->h,self->bits_per_value,&bits_per_value)) !=
        GRIB_SUCCESS)
     return err;
 
+  if(self->units_factor &&
+	 (grib_get_double_internal(a->parent->h,self->units_factor,&units_factor)== GRIB_SUCCESS)) {
+		  grib_set_double_internal(a->parent->h,self->units_factor,1.0);
+  }
+
+  if(self->units_bias &&
+	 (grib_get_double_internal(a->parent->h,self->units_bias,&units_bias)== GRIB_SUCCESS)) {
+		  grib_set_double_internal(a->parent->h,self->units_bias,0.0);
+	}
+
   if(n_vals==0){
     *len = 0;
     return GRIB_SUCCESS;
   }
 
-
   self->dirty=0;
 
   if((err = grib_get_double_internal(a->parent->h,self->reference_value, &reference_value)) !=
@@ -325,12 +346,6 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
 
   buf += grib_byte_offset(a);
 
-  if(*len < n_vals)
-  {
-    *len = (long) n_vals;
-    return GRIB_ARRAY_TOO_SMALL;
-  }
-
   Assert(((bits_per_value*n_vals)/8) < (1<<29));
   /*ensure that the bit pointer is not overflown*/
 
@@ -341,9 +356,18 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
 
   *len = (long) n_vals;
 
+  if (units_factor != 1.0) {
+	if (units_bias != 0.0)
+		for (i=0;i<n_vals;i++) val[i]=val[i]*units_factor+units_bias;
+	else 
+		for (i=0;i<n_vals;i++) val[i]*=units_factor;
+  } else if (units_bias != 0.0)
+		for (i=0;i<n_vals;i++) val[i]+=units_bias;
+
   return err;
 }
 
+
 static int pack_double(grib_accessor* a, const double* val, size_t *len)
 {
   grib_accessor_data_simple_packing* self =  (grib_accessor_data_simple_packing*)a;
@@ -355,20 +379,16 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
   long   binary_scale_factor = 0;
   long   bits_per_value = 0;
   long   decimal_scale_factor = 0;
-  double d = 1;
-  size_t buflen = 0;
-  unsigned char*  buf = NULL;
-  unsigned char*  encoded = NULL;
+  double decimal = 1;
   double max = 0;
   double min = 0;
   double unscaled_max = 0;
   double unscaled_min = 0;
-  double divisor = 1;
-  long off = 0;
   long imin,imax;
   double f=0;
   double range=0;
   double minrange=0,maxrange=0;
+  long changing_precision=0;
   grib_context* c=a->parent->h->context;
 
   if (*len ==0) return GRIB_NO_VALUES;
@@ -381,10 +401,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
      != GRIB_SUCCESS)
     return err;
 
-  if(*len == 0) {
-    grib_buffer_replace(a, NULL, 0,1,1);
-    return GRIB_SUCCESS;
-  }
+  if(*len == 0) return GRIB_SUCCESS;
 
   self->dirty=1;
 
@@ -394,11 +411,11 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
     if (val[i] > max ) max = val[i];
     if (val[i] < min ) min = val[i];
   }
-  d = grib_power(decimal_scale_factor,10) ;
+  decimal = grib_power(decimal_scale_factor,10) ;
   unscaled_min=min;
   unscaled_max=max;
-  min*=d;
-  max*=d;
+  min*=decimal;
+  max*=decimal;
 
   /* constant field only reference_value is set and bits_per_value=0 */
   if(max == min && c->gribex_mode_on==0) {
@@ -407,7 +424,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
         grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
          "unable to find nearest_smaller_value of %g for %s",min,self->reference_value);
         exit(GRIB_INTERNAL_ERROR);
-  }
+    }
     if((err = grib_set_double_internal(a->parent->h,self->reference_value, reference_value)) !=
       GRIB_SUCCESS)
       return err;
@@ -421,21 +438,25 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
       /* Make sure we can decode it again */
       double ref = 1e-100;
       grib_get_double_internal(a->parent->h,self->reference_value,&ref);
+	  if (ref != reference_value) 
+	    printf("%.20e  !=  %.20e",ref,reference_value);
       Assert(ref == reference_value);
     }
 
-    grib_buffer_replace(a, NULL, 0,1,1);
-
-    return GRIB_SUCCESS;
+     return GRIB_CONSTANT_FIELD;
   }
 
   if((err = grib_get_long_internal(a->parent->h,self->binary_scale_factor, &binary_scale_factor))
       != GRIB_SUCCESS)
     return err;
 
+  if((err = grib_get_long_internal(a->parent->h,self->changing_precision, &changing_precision))
+      != GRIB_SUCCESS)
+    return err;
+
   /* the packing parameters are not properly defined
   this is a safe way of fixing the problem */
-  if ( bits_per_value==0 && decimal_scale_factor==0) {
+  if ( changing_precision==0 && bits_per_value==0 && decimal_scale_factor==0) {
     grib_context_log(a->parent->h->context,GRIB_LOG_WARNING,
                      "%s==0 and %s==0 (setting %s=24)",
                      self->bits_per_value,
@@ -465,7 +486,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
          "unable to find nearest_smaller_value of %g for %s",min,self->reference_value);
         exit(GRIB_INTERNAL_ERROR);
     }
-    divisor=1;
+    /* divisor=1; */
   } else {
     int last=127;
     /* bits_per_value is given and decimal_scale_factor
@@ -473,7 +494,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
     */
     if (max == min) {
       binary_scale_factor=0;
-      divisor=1;
+      /* divisor=1; */
       if (grib_get_nearest_smaller_value(a->parent->h,self->reference_value,min,&reference_value)
         !=GRIB_SUCCESS) {
         grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
@@ -491,16 +512,16 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 
       while (range<minrange) {
         decimal_scale_factor+=1;
-        d*=10;
-        min=unscaled_min*d;
-        max=unscaled_max*d;
+        decimal*=10;
+        min=unscaled_min*decimal;
+        max=unscaled_max*decimal;
         range=(max-min);
       }
       while (range>maxrange) {
         decimal_scale_factor-=1;
-        d/=10;
-        min=unscaled_min*d;
-        max=unscaled_max*d;
+        decimal/=10;
+        min=unscaled_min*decimal;
+        max=unscaled_max*decimal;
         range=(max-min);
       }
 
@@ -512,24 +533,9 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
       }
 
       binary_scale_factor = grib_get_binary_scale_fact(max,reference_value,bits_per_value);
-
-      divisor = grib_power(-binary_scale_factor,2);
     }
   }
 
-  buflen = (((bits_per_value*n_vals)+7)/8)*sizeof(unsigned char);
-  buf = grib_context_buffer_malloc_clear(a->parent->h->context,buflen);
-  encoded = buf;
-
-  /*
-  printf("min=%g reference_value=%g min-ref=%g",min,reference_value,min-reference_value);
-  */
-
-  grib_encode_double_array(n_vals,val,bits_per_value,reference_value,d,divisor,encoded,&off);
-
-  grib_context_log(a->parent->h->context, GRIB_LOG_DEBUG,
-      "grib_accessor_data_simple_packing : pack_double : packing %s, %d values", a->name, n_vals);
-
 
   if((err = grib_set_double_internal(a->parent->h,self->reference_value, reference_value)) !=
       GRIB_SUCCESS)
@@ -541,11 +547,15 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
     grib_get_double_internal(a->parent->h,self->reference_value,&ref);
     if(ref!=reference_value)
     {
-      printf("ref=%g reference_value=%g diff=%g\n",ref,reference_value,ref-reference_value);
+      printf("ref=%g reference_value=%g diff=%g\n",
+             ref,reference_value,ref-reference_value);
       Assert(ref == reference_value);
     }
   }
 
+  if((err = grib_set_long_internal(a->parent->h,self->changing_precision, 0)) !=
+     GRIB_SUCCESS)
+    return err;
   if((err = grib_set_long_internal(a->parent->h,self->binary_scale_factor, binary_scale_factor)) !=
      GRIB_SUCCESS)
     return err;
@@ -553,9 +563,6 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
      != GRIB_SUCCESS)
     return err;
 
-  grib_buffer_replace(a, buf, buflen,1,1);
-
-  grib_context_buffer_free(a->parent->h->context,buf);
 
   return GRIB_SUCCESS;
 
