@@ -18,7 +18,7 @@
 
    START_CLASS_DEF
    CLASS      = accessor
-   IMPLEMENTS = unpack_long;pack_long
+   IMPLEMENTS = unpack_long;pack_long; clear
    IMPLEMENTS = unpack_double;pack_double;unpack_double_element
    IMPLEMENTS = unpack_string;pack_string
    IMPLEMENTS = unpack_bytes;pack_bytes
@@ -68,6 +68,7 @@ static size_t preferred_size(grib_accessor*,int);
 static grib_accessor* next(grib_accessor*, int);
 static int compare(grib_accessor*, grib_accessor*);
 static int unpack_double_element(grib_accessor*,size_t i, double* val);
+static int clear(grib_accessor*);
 
 typedef struct grib_accessor_gen {
     grib_accessor          att;
@@ -109,7 +110,8 @@ static grib_accessor_class _grib_accessor_class_gen = {
     0,      /* nearest_smaller_value */
     &next,                       /* next accessor    */
     &compare,                    /* compare vs. another accessor   */
-    &unpack_double_element,             /* unpack only ith value          */
+    &unpack_double_element,     /* unpack only ith value          */
+    &clear,             		/* clear          */
 };
 
 
@@ -127,6 +129,8 @@ static void init(grib_accessor* a,const long len, grib_arguments* param)
 
  if (a->flags & GRIB_ACCESSOR_FLAG_TRANSIENT) {
     a->length = 0;
+	if (a->vvalue != NULL)
+	        grib_context_free_persistent(a->parent->h->context, a->vvalue);
     a->vvalue = grib_context_malloc_clear_persistent(a->parent->h->context,sizeof(grib_virtual_value));
     a->vvalue->type=grib_accessor_get_native_type(a);
     a->vvalue->length=len;
@@ -225,6 +229,17 @@ static int unpack_bytes(grib_accessor* a, unsigned char* val, size_t *len)
   return GRIB_SUCCESS;
 }
 
+static int clear(grib_accessor* a)
+{
+  unsigned char* buf = a->parent->h->buffer->data;
+  long length = grib_byte_count(a);
+  long offset = grib_byte_offset(a);
+
+  memset(buf + offset,0,length);
+
+  return GRIB_SUCCESS;
+}
+
 static int  unpack_long   (grib_accessor* a, long*  v, size_t *len){
 
   if(a->cclass->unpack_double && a->cclass->unpack_double != &unpack_double)
