@@ -26,11 +26,6 @@ extern "C" {
 #include <string.h>
 #include <sys/types.h>
 
-#ifndef HAVE_FSEEKO
-#define off_t long
-#endif
-
-
 #define GRIB_API_VERSION (GRIB_API_MAJOR_VERSION*10000+GRIB_API_MINOR_VERSION*100+GRIB_API_REVISION_VERSION)
 
 /* LOG MODES
@@ -66,9 +61,11 @@ Log mode for information for processiong information
 #define GRIB_TYPE_MISSING 7
 
 /* Missing values */
-#define GRIB_MISSING_LONG   0x80000001
+/* #define GRIB_MISSING_LONG   0x80000001*/
+#define GRIB_MISSING_LONG 0xffffffff
 #define GRIB_MISSING_DOUBLE -1e+100
 
+
 /* Dump option flags*/
 #define GRIB_DUMP_FLAG_READ_ONLY    (1<<0)
 #define GRIB_DUMP_FLAG_DUMP_OK      (1<<1)
@@ -128,7 +125,11 @@ Log mode for information for processiong information
 \see grib_keys_iterator_new */
 #define GRIB_KEYS_ITERATOR_SKIP_FUNCTION           (1<<6)
 
-typedef struct grib_values {
+typedef struct grib_key_value_list grib_key_value_list;
+
+typedef struct grib_values grib_values;
+         
+struct grib_values {
   const char* name;
   int         type;
   long        long_value;
@@ -137,7 +138,8 @@ typedef struct grib_values {
   int         error;
   int         has_value;
   int         equal;
-} grib_values;
+  grib_values* next;
+} ;
 
 /*! Grib handle,   structure giving access to parsed grib values by keys
     \ingroup grib_handle
@@ -343,7 +345,7 @@ int grib_count_in_file(grib_context* c, FILE* f,int* n);
 grib_handle* grib_handle_new_from_file     (grib_context* c, FILE* f, int* error)        ;
 
 /**
-*  Create a handle from a user message. The message will not be freed at the end.
+*  Create a handle from a user message in memory. The message will not be freed at the end.
 *  The message will be copied as soon as a modification is needed.
 *
 * @param c           : the context from which the handle will be created (NULL for default context)
@@ -353,8 +355,19 @@ grib_handle* grib_handle_new_from_file     (grib_context* c, FILE* f, int* error
 */
 grib_handle* grib_handle_new_from_message(grib_context* c, void* data, size_t data_len);
 
+/**
+*  Create a handle from a user message in memory. The message will not be freed at the end.
+*  The message will be copied as soon as a modification is needed.
+*  This function works also with multi field messages.
+*
+* @param c           : the context from which the handle will be created (NULL for default context)
+* @param data        : the actual message
+* @param data_len    : the length of the message in number of bytes
+* @param error       : error code
+* @return            the new handle, NULL if the message is invalid or a problem is encountered
+*/
 grib_handle* grib_handle_new_from_multi_message(grib_context* c,void** data,
-                                                size_t *buflen,int* error);
+                                                size_t *data_len,int* error);
 
 /**
 *  Create a handle from a user message. The message is copied and will be freed with the handle
@@ -721,6 +734,17 @@ int grib_get_double_array (grib_handle* h, const char* key, double* vals,
 
 
 /*   setting      data         */
+/**
+*  Copy the keys belonging to a given namespace from a source handle to a destination handle
+*
+*
+* @param dest      : destination handle
+* @param name      : namespace
+* @param src       : source handle
+* @return          0 if OK, integer value on error
+*/
+int grib_copy_namespace(grib_handle* dest, const char* name, grib_handle* src);
+  
 /**
 *  Set a long value from a key. If several keys of the same name are present, the last one is set
 *  @see  grib_get_long
@@ -1172,6 +1196,11 @@ int grib_datetime_to_julian(long year, long month, long day, long hour, long min
 long grib_julian_to_date(long jdate);
 long grib_date_to_julian(long ddate);
 
+/* read products */
+int wmo_read_any_from_file(FILE *f, void *buffer, size_t *len);
+int wmo_read_any_from_stream(void *stream_data, long (*stream_proc )(void *, void *buffer, long len ), void *buffer, size_t *len);
+void *wmo_read_any_from_file_malloc(FILE *f, int *err);
+
 
 #ifdef __cplusplus
 }
@@ -1268,4 +1297,16 @@ long grib_date_to_julian(long ddate);
 #define GRIB_WRONG_GRID		-42
 /** End of index reached */
 #define GRIB_END_OF_INDEX		-43
+/** Null index */
+#define GRIB_NULL_INDEX		-44
+/** End of ressource reached when reading message */
+#define GRIB_PREMATURE_END_OF_FILE		-45
+/** An internal array is too small */
+#define GRIB_INTERNAL_ARRAY_TOO_SMALL		-46
+/** Message is too large for the current architecture */
+#define GRIB_MESSAGE_TOO_LARGE		-47
+/** Constant field */
+#define GRIB_CONSTANT_FIELD		-48
+/** Switch unable to find a matching case */
+#define GRIB_SWITCH_NO_MATCH		-49
 #endif
