@@ -94,7 +94,6 @@ extern int pthread_mutexattr_settype(pthread_mutexattr_t* attr,int type);
 #define Assert(a) {if(!(a)) grib_fail(#a,__FILE__,__LINE__);}
 
 #include "grib_api.h"
-#include "grib_associative_array.h"
 
 #define GRIB_UNKNOWN_VALUE   -9999.999
 
@@ -104,6 +103,8 @@ extern int pthread_mutexattr_settype(pthread_mutexattr_t* attr,int type);
 #define MAX_NUM_ACCESSORS 1000
 #define MAX_NUM_CONCEPTS 1000
 
+#define GRIB_NAMESPACE      10
+
 #define GRIB_MY_BUFFER      0
 #define GRIB_USER_BUFFER    1
 
@@ -190,6 +191,7 @@ typedef  int   (*accessor_pack_double_proc)              (grib_accessor*, const
 typedef  int   (*accessor_pack_string_proc)              (grib_accessor*, const char*,   size_t *len);
 typedef  int   (*accessor_pack_bytes_proc)               (grib_accessor*, const unsigned char*, size_t *len);
 typedef  int   (*accessor_pack_expression_proc)           (grib_accessor*, grib_expression*);
+typedef  int   (*accessor_clear_proc)           (grib_accessor*);
 
 typedef  void  (*accessor_init_class_proc)               (grib_accessor_class*);
 
@@ -213,6 +215,21 @@ typedef struct second_order_packed second_order_packed;
 typedef  void  grib_expression_visit_proc                (void* udata, grib_expression *e);
 
 
+
+struct grib_key_value_list {
+  const char* name;
+  int         type;
+  int         size;
+  long*       long_value;
+  double*     double_value;
+  grib_key_value_list* namespace_value;
+  char* string_value;
+  int         has_value;
+  int         error;
+  grib_key_value_list* next;
+} ;
+
+
 struct second_order_packed {
   unsigned long nbits_per_widths;
   unsigned long nbits_per_group_size;
@@ -267,6 +284,14 @@ struct grib_action
     unsigned long            flags;
     char                    *defaultkey; /** name of the key used as default if not found  */
 	grib_arguments*         default_value; /** default expression as in .def file */
+	int                     priority;     /** higher priority keys must be set before **/
+};
+
+typedef struct grib_accessor_list grib_accessor_list;
+
+struct grib_accessor_list {
+	grib_accessor* accessor;
+	grib_accessor_list* next;
 };
 
 
@@ -366,6 +391,7 @@ struct grib_accessor
   grib_accessor          *same;      /** < accessors with the same name */
   long                   loop;      /** < used in lists */
   grib_virtual_value*    vvalue;    /** < virtual value used when transient flag on **/
+  int                    priority;     /** higher priority keys must be set before **/
 
 };
 
@@ -383,6 +409,8 @@ struct grib_accessor
 #define GRIB_ACCESSOR_FLAG_DATA             (1<<11)
 #define GRIB_ACCESSOR_FLAG_REQUIRED         (1<<12)
 #define GRIB_ACCESSOR_FLAG_TRANSIENT        (1<<13)
+#define GRIB_ACCESSOR_FLAG_STRING_TYPE      (1<<14)
+#define GRIB_ACCESSOR_FLAG_LONG_TYPE        (1<<15)
 
 /**
 *  a section accessor
@@ -550,7 +578,6 @@ struct grib_handle
     grib_handle*            main;           /** Used during reparsing */
     grib_handle*            kid;           /** Used during reparsing */
     grib_loader*            loader;        /** Used during reparsing */
-    grib_associative_array* key_values;    /** Used when creating a new grib */
     int                     values_stack;
     const grib_values*      values[MAX_SET_VALUES];       /** Used when setting multiple values at once */
     size_t                  values_count[MAX_SET_VALUES];  /** Used when setting multiple values at once */
@@ -565,6 +592,7 @@ struct grib_handle
     char* section_offset[MAX_NUM_SECTIONS];
     char* section_length[MAX_NUM_SECTIONS];
     int sections_count;
+    off_t offset;
 };
 
 struct grib_multi_handle {
@@ -627,12 +655,14 @@ struct grib_accessor_class
     accessor_next_proc              next;
     accessor_compare_proc           compare;
     accessor_unpack_double_element_proc     unpack_double_element;
+    accessor_clear_proc             clear;
 };
 
 typedef struct grib_multi_support grib_multi_support;
 
 struct grib_multi_support {
     FILE*                           file;
+    size_t                          offset;
     unsigned char*                  message;
     size_t                          message_length;
     unsigned char*                  sections[8];
@@ -722,8 +752,10 @@ struct grib_context
     grib_itrie*                     concepts_index;
     int                             concepts_count;
     grib_concept_value*             concepts[MAX_NUM_CONCEPTS];
-    int                             ieee_packing;
     grib_string*                    def_files;
+    
+    grib_string*                    blacklist;
+    int                             ieee_packing;
 #if GRIB_PTHREADS
     pthread_mutex_t                 mutex;
 #endif
@@ -880,6 +912,8 @@ struct grib_index {
   grib_context* context;
   grib_index_key* keys;
   int rewind;
+  int orderby;
+  grib_index_key* orederby_keys;
   grib_field_tree* fields;
   grib_field_list* fieldset;
   grib_field_list* current;
@@ -949,6 +983,13 @@ struct grib_rule {
   grib_rule_entry  *entries;
 };
 
+typedef struct grib_case grib_case;
+
+struct grib_case {
+   grib_arguments* values;
+   grib_action* action;
+   grib_case* next;
+};
 
 /* ----------*/
 
@@ -966,7 +1007,7 @@ struct grib_codetable {
   code_table_entry entries[1];
 };
 
-#if GRIB_TIMER
+#if 0
 typedef struct grib_timer {
 
     struct timeval start_;
@@ -987,6 +1028,27 @@ typedef struct grib_timer {
 } grib_timer;
 #endif
 
+typedef struct j2k_encode_helper {
+
+  size_t           buffer_size;
+
+  long             width;
+  long             height;
+  long             bits_per_value;
+
+  float            compression;
+
+  long             no_values;
+  const double    *values;
+  double           reference_value;
+  double           divisor;
+  double           decimal;
+
+  long            jpeg_length;
+  unsigned char*  jpeg_buffer;
+
+} j2k_encode_helper;
+
 
 #include "grib_api_prototypes.h"
 
