@@ -12,7 +12,7 @@ void grib_dump_action_tree(grib_context *ctx, FILE *out);
 void grib_xref_action_branch(FILE *out, grib_action *a, const char *path);
 
 /* action_class_alias.c */
-grib_action *grib_action_create_alias(grib_context *context, const char *name, const char *arg1, const char *name_space);
+grib_action *grib_action_create_alias(grib_context *context, const char *name, const char *arg1, const char *name_space, int flags, int priority);
 
 /* action_class_gen.c */
 grib_action *grib_action_create_gen(grib_context *context, const char *name, const char *op, const long len, grib_arguments *params, grib_arguments *default_value, int flags, const char *name_space);
@@ -20,6 +20,16 @@ grib_action *grib_action_create_gen(grib_context *context, const char *name, con
 /* action_class_if.c */
 grib_action *grib_action_create_if(grib_context *context, grib_expression *expression, grib_action *block_true, grib_action *block_false);
 
+/* action_class_switch.c */
+grib_action *grib_action_create_switch(grib_context *context, grib_arguments *args, grib_case *Case, grib_action *Default);
+grib_case *grib_case_new(grib_context *c, grib_arguments *values, grib_action *action);
+
+/* grib_accessor_class_g1fcperiod.c */
+
+/* grib_accessor_class_g1end_of_interval_monthly.c */
+
+/* grib_accessor_class_mars_param.c */
+
 /* action_class_section.c */
 
 /* action_class_list.c */
@@ -41,16 +51,19 @@ grib_action *grib_action_create_remove(grib_context *context, grib_arguments *ar
 grib_action *grib_action_create_assert(grib_context *context, grib_expression *expression);
 
 /* action_class_template.c */
-grib_action *grib_action_create_template(grib_context *context, const char *name, const char *arg1);
+grib_action *grib_action_create_template(grib_context *context, int nofail, const char *name, const char *arg1);
 
 /* action_class_trigger.c */
 grib_action *grib_action_create_trigger(grib_context *context, grib_arguments *args, grib_action *block);
 
 /* action_class_when.c */
-grib_action *grib_action_create_when(grib_context *context, grib_expression *expression, grib_action *block);
+grib_action *grib_action_create_when(grib_context *context, grib_expression *expression, grib_action *block_true, grib_action *block_false);
+
+/* action_class_late_when.c */
+grib_action *grib_action_create_late_when(grib_context *context, grib_expression *expression, grib_action *block_true, grib_action *block_false);
 
 /* action_class_concept.c */
-grib_action *grib_action_create_concept(grib_context *context, const char *name, grib_concept_value *concept, const char *basename, const char *name_space, const char *defaultkey, const char *masterDir, const char *localDir, int flags);
+grib_action *grib_action_create_concept(grib_context *context, const char *name, grib_concept_value *concept, const char *basename, const char *name_space, const char *defaultkey, const char *masterDir, const char *localDir, int flags, int nofail);
 const char *grib_concept_evaluate(grib_handle *h, grib_action *act);
 int grib_concept_apply(grib_handle *h, grib_action *act, const char *name);
 
@@ -67,7 +80,7 @@ grib_action *grib_action_create_set_iarray(grib_context *context, const char *na
 grib_action *grib_action_create_noop(grib_context *context, const char *fname);
 
 /* action_class_write.c */
-grib_action *grib_action_create_write(grib_context *context, const char *name);
+grib_action *grib_action_create_write(grib_context *context, const char *name, int append);
 
 /* action_class_print.c */
 grib_action *grib_action_create_print(grib_context *context, const char *name, char *outname);
@@ -81,6 +94,7 @@ grib_action *grib_action_create_modify(grib_context *context, const char *name,
 /* grib_accessor.c */
 void grib_print_accessor(grib_accessor *a, grib_dumper *f);
 int grib_pack_missing(grib_accessor *a);
+int grib_pack_zero(grib_accessor *a);
 int grib_is_missing_internal(grib_accessor *a);
 int grib_pack_double(grib_accessor *a, const double *v, size_t *len);
 int grib_pack_expression(grib_accessor *a, grib_expression *e);
@@ -131,12 +145,16 @@ void grib_iarray_delete(grib_context *c, grib_iarray *v);
 
 /* grib_accessor_class_array.c */
 
+/* grib_accessor_class_assert.c */
+
 /* grib_accessor_class_ascii.c */
 
 /* grib_accessor_class_bit.c */
 
 /* grib_accessor_class_bitmap.c */
 
+/* grib_accessor_class_bits.c */
+
 /* grib_accessor_class_bits_per_value.c */
 
 /* grib_accessor_class_g1bitmap.c */
@@ -151,6 +169,10 @@ void grib_iarray_delete(grib_context *c, grib_iarray *v);
 
 /* grib_accessor_class_budgdate.c */
 
+/* grib_accessor_class_validity_date.c */
+
+/* grib_accessor_class_validity_time.c */
+
 /* grib_accessor_class_bytes.c */
 
 /* grib_accessor_class.c */
@@ -168,7 +190,6 @@ void grib_update_paddings(grib_section *s);
 /* grib_accessor_class_codeflag.c */
 
 /* grib_accessor_class_codetable.c */
-int grib_load_codetable(grib_context *c, const char *filename, const char *recomposed_name, size_t size, grib_codetable *t);
 void grib_codetable_delete(grib_context *c);
 
 /* grib_accessor_class_codetable_units.c */
@@ -181,6 +202,8 @@ void grib_codetable_delete(grib_context *c);
 
 /* grib_accessor_class_double.c */
 
+/* grib_accessor_class_element.c */
+
 /* grib_accessor_class_evaluate.c */
 
 /* grib_accessor_class_g1area.c */
@@ -189,14 +212,16 @@ void grib_codetable_delete(grib_context *c);
 
 /* grib_accessor_class_g1monthlydate.c */
 
+/* grib_accessor_class_library_version.c */
+
 /* grib_accessor_class_when.c */
 
+/* grib_accessor_class_late_when.c */
+
 /* grib_accessor_class_g1verificationdate.c */
 
 /* grib_accessor_class_g1day_of_the_year_date.c */
 
-/* grib_accessor_class_g1domain.c */
-
 /* grib_accessor_class_g1_half_byte_codeflag.c */
 
 /* grib_accessor_class_g1forecastmonth.c */
@@ -229,6 +254,8 @@ int grib_g1_step_apply_units(long *start, long *end, long *step_unit, long *P1,
 
 /* grib_accessor_class_gen.c */
 
+/* grib_accessor_class_gts_header.c */
+
 /* grib_accessor_class_julian_day.c */
 
 /* grib_accessor_class_latlonvalues.c */
@@ -295,6 +322,8 @@ grib_nearest *grib_nearest_new(grib_handle *h, int *error);
 
 /* grib_accessor_class_section.c */
 
+/* grib_accessor_class_step_in_units.c */
+
 /* grib_accessor_class_section_length.c */
 
 /* grib_accessor_class_g1_message_length.c */
@@ -326,6 +355,12 @@ int grib_get_g1_message_size(grib_handle *h, grib_accessor *tl, grib_accessor *s
 
 /* grib_accessor_class_data_simple_packing.c */
 
+/* grib_accessor_class_count_missing.c */
+
+/* grib_accessor_class_data_sh_packed.c */
+
+/* grib_accessor_class_data_sh_unpacked.c */
+
 /* grib_accessor_class_number_of_values_data_raw_packing.c */
 
 /* grib_accessor_class_data_g1simple_packing.c */
@@ -386,6 +421,18 @@ second_order_packed *grib_get_second_order_groups(grib_context *c, const unsigne
 
 /* grib_accessor_class_md5.c */
 
+/* grib_jasper_encoding.c */
+int grib_jasper_decode(grib_context *c, unsigned char *buf, size_t *buflen, double *values, size_t *no_values);
+int grib_jasper_encode(grib_context *c, j2k_encode_helper *helper);
+int grib_jasper_decode(grib_context *c, unsigned char *buf, size_t *buflen, double *val, size_t *n_vals);
+int grib_jasper_encode(grib_context *c, j2k_encode_helper *helper);
+
+/* grib_openjpeg_encoding.c */
+int grib_openjpeg_encode(grib_context *c, j2k_encode_helper *helper);
+int grib_openjpeg_decode(grib_context *c, unsigned char *buf, size_t *buflen, double *val, size_t *n_vals);
+int grib_openjpeg_decode(grib_context *c, unsigned char *buf, size_t *buflen, double *val, size_t *n_vals);
+int grib_openjpeg_encode(grib_context *c, j2k_encode_helper *helper);
+
 /* action_class_set_missing.c */
 grib_action *grib_action_create_set_missing(grib_context *context, const char *name);
 
@@ -420,6 +467,8 @@ void grib_index_rewind(grib_index *index);
 
 /* grib_accessor_class_statistics.c */
 
+/* grib_accessor_class_statistics_spectral.c */
+
 /* grib_accessor_class_unsigned.c */
 
 /* grib_accessor_class_vector.c */
@@ -437,45 +486,6 @@ void grib_gaussian_reduced_sub_row(long pl, double lon_first, double lon_last, l
 int grib_lookup_long_from_handle(grib_context *gc, grib_loader *loader, const char *name, long *value);
 int grib_init_accessor_from_handle(grib_loader *loader, grib_accessor *ga, grib_arguments *default_value);
 
-/* grib_loader_from_array.c */
-int grib_lookup_long_from_array(grib_context *gc, grib_loader *loader, const char *name, long *lvalue);
-int grib_init_accessor_from_array(grib_loader *loader, grib_accessor *ga, grib_arguments *default_value);
-
-/* grib_associative_array.c */
-grib_runtime_type *grib_runtimetype_new(grib_context *gc);
-void grib_runtimetype_destroy(grib_runtime_type *rttv);
-int grib_runtimetype_copy(grib_runtime_type *copy, grib_runtime_type *original);
-int grib_runtimetype_get_long(grib_runtime_type *rttv, long **pv, size_t *size);
-int grib_runtimetype_get_double(grib_runtime_type *rttv, double **pv, size_t *size);
-int grib_runtimetype_get_char(grib_runtime_type *rttv, char **pv, size_t *size);
-int grib_runtimetype_get(grib_runtime_type *rttv, void **pv, size_t *size);
-int grib_runtimetype_get_type(grib_runtime_type *rttv, int *type);
-int grib_runtimetype_set_long(grib_runtime_type *rttv, long *v, size_t size);
-int grib_runtimetype_set_double(grib_runtime_type *rttv, double *v, size_t size);
-int grib_runtimetype_set_char(grib_runtime_type *rttv, char *v, size_t size);
-int grib_runtimetype_set(grib_runtime_type *rttv, void *v, int type, size_t size);
-grib_associative_array_el *grib_associative_array_el_new(grib_context *gc);
-void grib_associative_array_el_destroy(grib_associative_array_el *el);
-grib_associative_array *grib_associative_array_new(grib_context *gc);
-void grib_associative_array_destroy(grib_associative_array *ar);
-int grib_associative_array_set(grib_associative_array *ar, char *key, grib_runtime_type *value);
-int grib_associative_array_set_long(grib_associative_array *ar, char *key, long *v, size_t size);
-int grib_associative_array_set_double(grib_associative_array *ar, char *key, double *v, size_t size);
-int grib_associative_array_set_char(grib_associative_array *ar, char *key, char *v, size_t size);
-int grib_associative_array_push(grib_associative_array *ar, char *key, grib_runtime_type *value);
-int grib_associative_array_push_long(grib_associative_array *ar, char *key, long *lvalue, size_t size);
-int grib_associative_array_push_double(grib_associative_array *ar, char *key, double *dvalue, size_t size);
-int grib_associative_array_push_char(grib_associative_array *ar, char *key, char *cvalue, size_t size);
-int grib_associative_array_get(grib_associative_array *ar, const char *key, grib_runtime_type **value);
-int grib_associative_array_get_long(grib_associative_array *ar, const char *key, long **value, size_t *size);
-int grib_associative_array_get_double(grib_associative_array *ar, const char *key, double **value, size_t *size);
-int grib_associative_array_get_char(grib_associative_array *ar, const char *key, char **value, size_t *size);
-int grib_file_to_array(FILE *in, grib_associative_array *ar);
-
-/* grib_loader_from_file.c */
-int grib_lookup_long_from_file(grib_context *gc, grib_loader *loader, const char *name, long *value);
-int grib_init_accessor_from_file(grib_loader *loader, grib_accessor *ga, grib_arguments *default_value);
-
 /* grib_bits.c */
 unsigned long grib_decode_unsigned_byte_long(const unsigned char *p, long o, int l);
 long grib_decode_signed_long(const unsigned char *p, long o, int l);
@@ -599,7 +609,6 @@ void grib_print_api_version(FILE *out);
 grib_context *grib_context_get_default(void);
 grib_context *grib_context_new(grib_context *parent);
 const char *grib_context_full_path(grib_context *c, const char *basename, char *full);
-int grib_init_definition_files_dir(grib_context *c);
 void grib_context_free(const grib_context *c, void *p);
 void grib_context_free_persistent(const grib_context *c, void *p);
 void grib_context_reset(grib_context *c);
@@ -665,9 +674,9 @@ grib_handle *grib_handle_new_from_samples(grib_context *c, const char *name);
 grib_handle *grib_handle_clone(grib_handle *h);
 grib_handle *grib_handle_new_from_message_copy(grib_context *c, const void *data, size_t size);
 grib_handle *grib_handle_new_from_partial_message_copy(grib_context *c, const void *data, size_t size);
+grib_handle *grib_handle_new_from_partial_message(grib_context *c, void *data, size_t buflen);
 grib_handle *grib_handle_new_from_message(grib_context *c, void *data, size_t buflen);
 grib_handle *grib_handle_new_from_multi_message(grib_context *c, void **data, size_t *buflen, int *error);
-grib_handle *grib_handle_new_from_partial_message(grib_context *c, void *data, size_t buflen);
 grib_handle *grib_handle_new_from_file(grib_context *c, FILE *f, int *error);
 grib_multi_handle *grib_multi_handle_new(grib_context *c);
 int grib_multi_handle_delete(grib_multi_handle *h);
@@ -677,13 +686,8 @@ int grib_get_partial_message(grib_handle *h, const void **msg, size_t *len, int
 int grib_get_partial_message_copy(grib_handle *h, void *message, size_t *len, int start_section);
 int grib_get_message_copy(grib_handle *h, void *message, size_t *len);
 int grib_get_message(grib_handle *h, const void **msg, size_t *size);
-grib_handle *grib_handle_new_from_input(grib_context *c, FILE *in);
-grib_handle *grib_handle_new_from_array(grib_associative_array *ar);
-int grib_load_from_text_file(grib_handle *h, FILE *in);
-int grib_set_type(grib_handle *h, const char *type);
+int grib_get_message_headers(grib_handle *h, const void **msg, size_t *size);
 grib_handle *grib_handle_new(grib_context *c);
-grib_handle *grib_handle_new_from_text_file(grib_context *c, FILE *f);
-int grib_handle_parse_key_values_array(grib_handle *h, grib_associative_array *kv);
 grib_action *grib_action_from_filter(const char *filter);
 int grib_handle_apply_action(grib_handle *h, grib_action *a);
 int grib_handle_prepare_action(grib_handle *h, grib_action *a);
@@ -702,10 +706,17 @@ void grib_math_delete(grib_context *c, grib_math *m);
 grib_math *grib_math_new(grib_context *c, const char *formula, int *err);
 
 /* grib_io.c */
-int grib_read_any_alloc(grib_context *c, unsigned char **data, size_t *data_len, unsigned char **message, size_t *message_len);
-int grib_read_any(grib_context *c, unsigned char **data, size_t *data_len, unsigned char **message, size_t *message_len);
-int grib_read_any_from_file(grib_context *ctx, void *file, unsigned char *mesg, size_t *size);
-int grib_read_any_from_file_alloc(grib_context *ctx, void *file, unsigned char **mesg, size_t *msg_len);
+int wmo_read_any_from_file(FILE *f, void *buffer, size_t *len);
+int wmo_read_grib_from_file(FILE *f, void *buffer, size_t *len);
+int wmo_read_bufr_from_file(FILE *f, void *buffer, size_t *len);
+int wmo_read_any_from_stream(void *stream_data, long (*stream_proc )(void *, void *buffer, long len ), void *buffer, size_t *len);
+void *wmo_read_any_from_file_malloc(FILE *f, int *err);
+void *wmo_read_grib_from_file_malloc(FILE *f, int *err);
+void *wmo_read_bufr_from_file_malloc(FILE *f, int *err);
+int grib_read_any_from_file_alloc(grib_context *ctx, FILE *f, void **buffer, size_t *length);
+int grib_read_any_from_file(grib_context *ctx, FILE *f, void *buffer, size_t *len);
+int grib_read_any_from_memory_alloc(grib_context *ctx, unsigned char **data, size_t *data_length, void **buffer, size_t *length);
+int grib_read_any_from_memory(grib_context *ctx, unsigned char **data, size_t *data_length, void *buffer, size_t *len);
 int grib_count_in_file(grib_context *c, FILE *f, int *n);
 
 /* grib_trie.c */
@@ -754,6 +765,7 @@ grib_action *grib_parse_file(grib_context *gc, const char *filename);
 int grib_type_to_int(char id);
 
 /* grib_query.c */
+grib_accessor *grib_find_accessor_fast(grib_handle *h, const char *name);
 grib_accessor *grib_find_accessor(grib_handle *h, const char *name);
 int grib_find_all_accessors(grib_handle *h, const char *name, search_all_callback_proc callback, void *data);
 
@@ -783,11 +795,13 @@ int grib_set_expression_internal(grib_handle *h, const char *name, grib_expressi
 int grib_set_long_internal(grib_handle *h, const char *name, long val);
 int grib_set_long(grib_handle *h, const char *name, long val);
 int grib_set_double_internal(grib_handle *h, const char *name, double val);
+int grib_copy_namespace(grib_handle *dest, const char *name, grib_handle *src);
 int grib_set_double(grib_handle *h, const char *name, double val);
 int grib_set_string_internal(grib_handle *h, const char *name, const char *val, size_t *length);
 int grib_set_string(grib_handle *h, const char *name, const char *val, size_t *length);
 int grib_set_bytes_internal(grib_handle *h, const char *name, const unsigned char *val, size_t *length);
 int grib_set_bytes(grib_handle *h, const char *name, const unsigned char *val, size_t *length);
+int grib_clear(grib_handle *h, const char *name);
 int grib_set_missing_internal(grib_handle *h, const char *name);
 int grib_set_missing(grib_handle *h, const char *name);
 int grib_is_missing(grib_handle *h, const char *name, int *err);
@@ -813,11 +827,15 @@ int _grib_get_double_array_internal(grib_handle *h, grib_accessor *a, double *va
 int grib_get_double_array_internal(grib_handle *h, const char *name, double *val, size_t *length);
 int grib_get_double_array(grib_handle *h, const char *name, double *val, size_t *length);
 int grib_get_size(grib_handle *h, const char *name, size_t *size);
+int grib_get_count(grib_handle *h, const char *name, size_t *size);
 int _grib_get_size(grib_handle *h, grib_accessor *a, size_t *size);
 int grib_get_offset(grib_handle *h, const char *key, size_t *val);
 int _grib_get_long_array_internal(grib_handle *h, grib_accessor *a, long *val, size_t buffer_len, size_t *decoded_length);
 int grib_get_long_array_internal(grib_handle *h, const char *name, long *val, size_t *length);
 int grib_get_long_array(grib_handle *h, const char *name, long *val, size_t *length);
+grib_key_value_list *grib_key_value_list_clone(grib_context *c, grib_key_value_list *list);
+void grib_key_value_list_delete(grib_context *c, grib_key_value_list *kvl);
+int grib_get_key_value_list(grib_handle *h, grib_key_value_list *list);
 int grib_get_values(grib_handle *h, grib_values *args, size_t count);
 int grib_set_values(grib_handle *h, grib_values *args, size_t count);
 int grib_get_nearest_smaller_value(grib_handle *h, const char *name, double val, double *nearest);
@@ -922,18 +940,3 @@ void cvs_MD5Update(struct cvs_MD5Context *ctx, unsigned char const *buf, unsigne
 void cvs_MD5Final(unsigned char digest[16 ], struct cvs_MD5Context *ctx);
 void cvs_MD5Transform(cvs_uint32 buf[4 ], const unsigned char inraw[64 ]);
 int main(int argc, char **argv);
-
-/* grib_timer.c */
-#if GRIB_TIMER
-double proc_cpu(void);
-char *timename(double t);
-grib_timer *grib_get_timer(grib_context *c, const char *name, const char *statname, int elapsed);
-int grib_timer_start(grib_timer *t);
-int grib_timer_stop(grib_timer *t, long total);
-double grib_timer_value(grib_timer *t);
-const char *bytename(double bytes);
-void grib_timer_print(grib_timer *t);
-void grib_timer_partial_rate(grib_timer *t, double start, long total);
-void grib_print_all_timers(void);
-void grib_reset_all_timers(void);
-#endif
