@@ -114,6 +114,102 @@ int grib_set_double_internal(grib_handle* h, const char* name, double val) {
   return GRIB_NOT_FOUND;
 }
 
+
+int grib_copy_namespace(grib_handle* dest, const char* name, grib_handle* src) {
+  const char* key=NULL;
+  int err=0;
+  int type;
+  size_t len;
+  char *sval = NULL;
+  unsigned char *uval = NULL;
+  double *dval = NULL;
+  long *lval = NULL;
+
+  grib_keys_iterator* iter=NULL;
+
+  if (!dest || !src) return GRIB_NULL_HANDLE;
+  
+  iter=grib_keys_iterator_new(src,0,(char*)name);
+  
+  if (!iter) {
+    printf("ERROR: unable to get iterator for %s\n",name );
+    exit(1);
+  }
+
+  while(grib_keys_iterator_next(iter))  {
+    key=grib_keys_iterator_get_name(iter);
+
+    if ( grib_is_missing(src,key,&err) && err == 0 && (err=grib_set_missing(dest,key))) {
+      if (err!=GRIB_SUCCESS) return err;
+      continue;
+    }
+  
+    if ((err=grib_get_native_type(src,key,&type))!=GRIB_SUCCESS) return err;
+  
+    if((err = grib_get_size(src,key,&len)) != GRIB_SUCCESS)  return err;
+
+    switch (type) {
+      case GRIB_TYPE_STRING:
+        len=512;
+        sval = grib_context_malloc(src->context,len*sizeof(char));
+        
+        if((err = grib_get_string(src,key,sval,&len)) != GRIB_SUCCESS)
+          return err;
+
+        if((err = grib_set_string(dest,key,sval,&len)) != GRIB_SUCCESS)
+          return err;
+
+        grib_context_free(src->context,sval);
+        break;
+
+      case GRIB_TYPE_LONG:
+        lval = grib_context_malloc(src->context,len*sizeof(long));
+
+        if((err = grib_get_long_array(src,key,lval,&len)) != GRIB_SUCCESS)
+          return err;
+
+        if((err = grib_set_long_array(dest,key,lval,len)) != GRIB_SUCCESS)
+          return err;
+
+        grib_context_free(src->context,lval);
+        break;
+
+      case GRIB_TYPE_DOUBLE:
+        dval = grib_context_malloc(src->context,len*sizeof(double));
+
+        if((err = grib_get_double_array(src,key,dval,&len)) != GRIB_SUCCESS)
+          return err;
+
+        if((err = grib_set_double_array(dest,key,dval,len)) != GRIB_SUCCESS)
+          return err;
+
+        break;
+
+      case GRIB_TYPE_BYTES:
+        if (len==0) len=512;
+        uval = grib_context_malloc(src->context,len*sizeof(unsigned char));
+        
+        if((err = grib_get_bytes(src,key,uval,&len)) != GRIB_SUCCESS)
+          return err;
+
+        if((err = grib_get_bytes(dest,key,uval,&len)) != GRIB_SUCCESS)
+          return err;
+
+        grib_context_free(src->context,uval);
+
+        break;
+
+      default:
+        break;
+    }
+
+  }
+  grib_keys_iterator_delete(iter);
+  
+  return err;
+}
+
+
 int grib_set_double(grib_handle* h, const char* name, double val) {
   int ret = GRIB_SUCCESS;
   grib_accessor* a =NULL;
@@ -220,6 +316,25 @@ int grib_set_bytes(grib_handle* h, const char* name, const unsigned char* val, s
   return GRIB_NOT_FOUND;
 }
 
+int grib_clear(grib_handle* h, const char* name)
+{
+  int ret=0;
+  grib_accessor* a =NULL;
+
+  a = grib_find_accessor(h, name);
+
+  if(a) {
+     if (a->length ==0) return 0;
+     if ((ret=grib_pack_zero(a)) != GRIB_SUCCESS)
+     grib_context_log(h->context,GRIB_LOG_ERROR,"unable to clear %s (%s)",
+                     name,grib_get_error_message(ret));
+    return ret;
+  }
+
+  /*grib_context_log(h->context,GRIB_LOG_ERROR,"unable to find accessor %s",name);*/
+  return GRIB_NOT_FOUND;
+}
+
 int grib_set_missing_internal(grib_handle* h, const char* name) {
   int ret=0;
   grib_accessor* a =NULL;
@@ -635,6 +750,19 @@ int grib_get_size(grib_handle* h, const char* name,size_t* size)
   }
   return GRIB_SUCCESS;
 }
+int grib_get_count(grib_handle* h, const char* name,size_t* size)
+{
+  grib_accessor* a = grib_find_accessor(h, name);
+  if(!a) return GRIB_NOT_FOUND;
+
+  *size = 0;
+  while(a) {
+    (*size)++;
+    a = a->same;
+  }
+  return GRIB_SUCCESS;
+}
+
 
 int _grib_get_size(grib_handle* h, grib_accessor* a,size_t* size)
 {
@@ -700,6 +828,117 @@ int grib_get_long_array(grib_handle* h, const char* name, long* val, size_t *len
   return _grib_get_long_array_internal(h,a,val,len,length);
 }
 
+static void grib_clean_key_value(grib_context* c,grib_key_value_list* kv) {
+  if (kv->long_value) grib_context_free(c,kv->long_value);
+  kv->long_value=NULL;
+  if (kv->double_value) grib_context_free(c,kv->double_value);
+  kv->double_value=NULL;
+  if (kv->string_value) grib_context_free(c,kv->string_value);
+  kv->string_value=NULL;
+  if (kv->namespace_value) grib_key_value_list_delete(c,kv->namespace_value);
+  kv->namespace_value=NULL;
+  kv->error=0;
+  kv->has_value=0;
+  kv->size=0;
+}
+
+static int grib_get_key_value(grib_handle* h,grib_key_value_list* kv) {
+  int ret=0;
+  size_t size=0;
+  grib_keys_iterator* iter=NULL;
+  grib_key_value_list* list=NULL;
+
+  if (kv->has_value) grib_clean_key_value(h->context,kv);
+  
+  ret=grib_get_size(h,kv->name,&size);
+  if (ret) {
+    kv->error=ret;
+    return ret;
+  }
+  if (size==0) size=512;
+  
+  switch (kv->type) {
+    case GRIB_TYPE_LONG:
+      kv->long_value=grib_context_malloc_clear(h->context,size*sizeof(long));
+      ret=grib_get_long_array(h,kv->name,kv->long_value,&size);
+      kv->error=ret;
+      break;
+    case GRIB_TYPE_DOUBLE:
+      kv->double_value=grib_context_malloc_clear(h->context,size*sizeof(double));
+      ret=grib_get_double_array(h,kv->name,kv->double_value,&size);
+      kv->error=ret;
+      break;
+    case GRIB_TYPE_STRING:
+      kv->string_value=grib_context_malloc_clear(h->context,size*sizeof(char));
+      ret=grib_get_string(h,kv->name,kv->string_value,&size);
+      kv->error=ret;
+      break;
+    case GRIB_TYPE_BYTES:
+      kv->string_value=grib_context_malloc_clear(h->context,size*sizeof(char));
+      ret=grib_get_bytes(h,kv->name,(unsigned char*)kv->string_value,&size);
+      kv->error=ret;
+      break;
+    case GRIB_NAMESPACE:
+      iter=grib_keys_iterator_new(h,0,(char*)kv->name);
+      list=grib_context_malloc_clear(h->context,sizeof(grib_key_value_list));
+      kv->namespace_value=list;
+      while(grib_keys_iterator_next(iter))
+      {
+        list->name=grib_keys_iterator_get_name(iter);
+        ret=grib_get_native_type(h,list->name,&(list->type));
+        ret=grib_get_key_value(h,list);
+        list->next=grib_context_malloc_clear(h->context,sizeof(grib_key_value_list));
+        list=list->next;
+      }
+      grib_keys_iterator_delete(iter);
+      break;
+
+    default:
+      ret=grib_get_native_type(h,kv->name,&(kv->type));
+      ret=grib_get_key_value(h,kv);
+      break;
+  }
+  kv->has_value=1;
+  return ret;
+}
+
+grib_key_value_list* grib_key_value_list_clone(grib_context* c,grib_key_value_list* list) {
+  grib_key_value_list* next=list;
+  grib_key_value_list* clone=grib_context_malloc_clear(c,sizeof(grib_key_value_list));
+  grib_key_value_list* p=clone;
+
+  while (next && next->name) {
+    p->name=grib_context_strdup(c,next->name);
+    p->type=next->type;
+    next=next->next;
+  }
+  return clone;
+}
+
+void grib_key_value_list_delete(grib_context* c,grib_key_value_list* kvl) {
+  grib_key_value_list* next=kvl;
+  grib_key_value_list* p=NULL;
+  while (next) {
+    p=next->next;
+    if (next->type == GRIB_NAMESPACE) 
+      grib_key_value_list_delete(c,next->namespace_value);
+    
+    grib_clean_key_value(c,next);
+    grib_context_free(c,next);
+    next=p;
+  }
+}
+
+int grib_get_key_value_list(grib_handle* h,grib_key_value_list* list) {
+  int ret=0;
+  grib_key_value_list* kvl=list;
+  while (kvl) {
+    ret=grib_get_key_value(h,kvl);
+    kvl=kvl->next;
+  }
+  return ret;
+}
+
 int grib_get_values(grib_handle* h,grib_values* args,size_t count) {
   int ret=0;
   int i=0;
@@ -799,9 +1038,9 @@ int grib_set_values(grib_handle* h,grib_values* args,size_t count)
           args[i].error =  GRIB_INVALID_ARGUMENT;
           break;
       }
-      if (args[i].error != GRIB_SUCCESS)
-        grib_context_log(h->context,GRIB_LOG_ERROR,"unable to set %s (%s)",
-                         args[i].name,grib_get_error_message(args[i].error));
+       /*if (args[i].error != GRIB_SUCCESS) 
+         grib_context_log(h->context,GRIB_LOG_ERROR,"unable to set %s (%s)", 
+                          args[i].name,grib_get_error_message(args[i].error)); */
     }
   }
 
@@ -814,10 +1053,10 @@ int grib_set_values(grib_handle* h,grib_values* args,size_t count)
   for(i = 0; i < count ; i++) {
     if(args[i].error != GRIB_SUCCESS)
     {
-      grib_context_log(h->context,GRIB_LOG_DEBUG,
+      grib_context_log(h->context,GRIB_LOG_ERROR,
           "grib_set_values[%d] %s (%d) failed: %s",  i, args[i].name, args[i].type,
           grib_get_error_message(args[i].error));
-      err=args[i].error;
+      err= err==GRIB_SUCCESS ? args[i].error : err;
     }
   }
 
