@@ -14,7 +14,7 @@ outfile=${infile}.1
 
 rm -f $outfile || true
 
-${tools_dir}grib_set -s short_name=2d $infile $outfile
+${tools_dir}grib_set -s shortName=2d $infile $outfile
 
 ${tools_dir}grib_compare -b indicatorOfParameter,paramId,shortName $infile $outfile > /dev/null
 
