@@ -9,15 +9,8 @@
 
 . ./include.sh
 
-for file in ${data_dir}/*.grib1
+for file in ${data_dir}/*.grib2 ${data_dir}/*.grib1
 do
-#echo $file
-${tools_dir}grib_debug $file 2> /dev/null > /dev/null
-done
-
-for file in ${data_dir}/*.grib2
-do
-#echo $file
 ${tools_dir}grib_debug $file 2> /dev/null > /dev/null
 done
 
