@@ -21,11 +21,11 @@ outfile2=${infile}_decimalPrecision_2
 
 ${tools_dir}grib_set -r -s bitsPerValue=0,decimalScaleFactor=1 $infile $outfile1
 
-${tools_dir}grib_compare -b dataLength,decimalScaleFactor,binaryScaleFactor,referenceValue,bitsPerValue $infile $outfile1 > /dev/null
+${tools_dir}grib_compare -b totalLength,section4Length,section7Length,dataLength,decimalScaleFactor,binaryScaleFactor,referenceValue,bitsPerValue $infile $outfile1 > /dev/null
 
 ${tools_dir}grib_set -s changeDecimalPrecision=1 $infile $outfile2
 
-${tools_dir}grib_compare -b dataLength,decimalScaleFactor,binaryScaleFactor,referenceValue,bitsPerValue $infile $outfile2 > /dev/null
+${tools_dir}grib_compare -b totalLength,section4Length,section7Length,dataLength,decimalScaleFactor,binaryScaleFactor,referenceValue,bitsPerValue $infile $outfile2 > /dev/null
 
 ${tools_dir}grib_cmp $outfile1 $outfile2
 
