@@ -12,7 +12,7 @@
 if [ -f ${data_dir}/geavg.t12z.pgrbaf108 ]
 then
 
-tmpdata=/tmp/grib_api.grib
+tmpdata=grib_api.grib
 
 rm -f $tmpdata || true
 
