@@ -12,6 +12,9 @@ echo "TEST: $0"
 
 data_dir=""
 
+# save current working dir
+save=`pwd`
+
 if [ -z "${data_dir}" ]
 then
   cd ../
@@ -39,5 +42,8 @@ fi
 
 #${tools_dir}grib_info
 
+# go back to current working dir
+cd $save
+
 set -u
 
