@@ -19,7 +19,7 @@ temp="temp.grib1_"
 
 # --- exclude babj as the way they encode grib 2 makes it
 # ---   problematic to convert to grib 1
-exclusion_pattern="none defined so far"
+exclusion_pattern="nothing"
 
 for file in `ls ${dir}/tigge_*.grib`
 do
@@ -41,7 +41,7 @@ if [ -z "$exclude" ]; then
 	  exit 1
 	fi
 
-	${tools_dir}grib_compare -c values ${temp} ${file} 2> /dev/null > /dev/null
+	${tools_dir}grib_compare -P -c values ${temp} ${file} 2> /dev/null > /dev/null
 
 	rm -f ${temp} || true
 fi
