@@ -222,8 +222,14 @@ void gaussian_grid(grib_handle* h)
 
 	if(missing(h,"numberOfPointsAlongAParallel"))
 	{
+		double  ee = 360.0 - 360.0/(4.0*n);
 		CHECK(west == 0.0);
-		CHECK(east == 360.0 - 360.0/(4.0*n));
+		
+		if(ee != east)
+			printf("east %g %g %g\n",east,ee,ee-east);
+			
+		CHECK(east == ee); 
+		CHECK(fabs(east-ee) < 1e-10);
 	}
 	else
 	{
@@ -484,20 +490,20 @@ static void statistical_process(grib_handle* h,const parameter* p,double min,dou
 		CHECK(eq(h,"minuteOfEndOfOverallTimeInterval",0));
 		CHECK(eq(h,"secondOfEndOfOverallTimeInterval",0));
 
-		if (!is_lam) CHECK((get(h,"marsEndStep") % 6) == 0);  /* Every six hours */
-		CHECK(get(h,"marsEndStep") <= 24*30);  
+		if (!is_lam) CHECK((get(h,"endStep") % 6) == 0);  /* Every six hours */
+		CHECK(get(h,"endStep") <= 24*30);  
 	}
 
 
 	if(get(h,"indicatorOfUnitOfTimeForTimeRangeOverWhichStatisticalProcessingIsDone") == 11)
 	{
 		/* Six hourly is OK */
-		CHECK(get(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone")*6 + get(h,"marsStartStep") == get(h,"marsEndStep"));
+		CHECK(get(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone")*6 + get(h,"startStep") == get(h,"endStep"));
 	}
 	else
 	{
 		CHECK(eq(h,"indicatorOfUnitOfTimeForTimeRangeOverWhichStatisticalProcessingIsDone",1)); /* Hours */
-		CHECK(get(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone") + get(h,"marsStartStep") == get(h,"marsEndStep"));
+		CHECK(get(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone") + get(h,"startStep") == get(h,"endStep"));
 	}
 
 }
@@ -517,7 +523,7 @@ static void six_hourly(grib_handle* h,const parameter* p,double min,double max)
 	else
 		CHECK(eq(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone",6));
 
-	CHECK(get(h,"marsEndStep") == get(h,"marsStartStep") + 6);
+	CHECK(get(h,"endStep") == get(h,"startStep") + 6);
 	check_range(h,p,min,max);
 }
 
@@ -530,15 +536,15 @@ static void three_hourly(grib_handle* h,const parameter* p,double min,double max
 	else
 		CHECK(eq(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone",3));
 
-	CHECK(get(h,"marsEndStep") == get(h,"marsStartStep") + 3);
+	CHECK(get(h,"endStep") == get(h,"startStep") + 3);
 	check_range(h,p,min,max);
 }
 
 static void from_start(grib_handle* h,const parameter* p,double min,double max)
 {
-	long step = get(h,"marsEndStep");
+	long step = get(h,"endStep");
 	statistical_process(h,p,min,max);
-	CHECK(eq(h,"marsStartStep",0));
+	CHECK(eq(h,"startStep",0));
 
 	if(step == 0)
 		CHECK(min == 0 && max == 0);
@@ -800,8 +806,8 @@ void verify(grib_handle* h)
 
 
 	if (!is_lam) CHECK((get(h,"step") % 6) == 0);
-	CHECK(ge(h,"marsStartStep",0));
-	CHECK(le(h,"marsEndStep",30*24));
+	CHECK(ge(h,"startStep",0));
+	CHECK(le(h,"endStep",30*24));
 
 
 	/* 2 = analysis or forecast , 3 = control forecast, 4 = perturbed forecast */
